/***
 * Area test #1
 *
 * Change Log:
 * 2003/11/25 vilda	created
 * 2003/11/29 vilda	updated
 * 2003/12/02 ghort	polished
 * 2004/04/25 pallas	really polished :-)
 * 2004/11/24 pallas	revamped and updated to match new assignment spec
 */

static const char *	desc =
	"Area test #1\n"
	"Tests whether virtual memory area management functions work. This\n"
	"is only a set of basic function calls with test of expected return\n"
	"values.\n\n"
	"In case of failure, read the test output carefully or look into\n"
	"the sources.\n\n";


#include "../../assignment-2.h"
#include "defs.h"



/**
 * result_name
 * @result:	result to translate
 *
 * A human-friendly result name.
 */
static char *
result_name (int result)
{
	char * name;

	switch (result) {
	case EOK:
		name = "EOK";
		break;

	case EINVAL:
		name = "EINVAL";
		break;

	case ENOMEM:
		name = "ENOMEM";
		break;

	default:
		name = "UNKNOWN";
	}

	return name;
} /* result_name */


/*
 * Fill a memory block with garbage.
 */
void
fillmem (u8 * ptr, size_t size)
{
	char chr;
	u8 * end;

	for (chr = 0, end = ptr + size; ptr < end; ptr++, chr++) {
		*ptr = chr;
	}
} /* fillmem */


/*
 * Vmalloc/vfree/vresize/vremap/vmerge/vsplit parameters.
 */
struct vparam {
	void *		from;		// vmalloc/vresize/vsplit @from
	size_t		size;		// vmalloc/vresize @size
	uint		flags;		// vmalloc @flags

	void *		area1;		// vmerge @area1
	void *		area2;		// vmerge @area2
	void *		split;		// vsplit @split
	void *		to;		// vremap @to

	int		result;		// expected result

	char *		desc;		// test description

	/*
	 * Test function to call with the parameters.
	 */
	int		(* test) (struct vparam * param);

	/*
	 * Evaluator fuction for the results.
	 */
	int		(* eval) (struct vparam * param, int result, ...);
}; /* vparam */


/**
 * Vmalloc evaluators.
 *
 * @param:	parameters of the vmalloc call
 * @result:	result of the call
 * @from:	resulting start address
 *
 * The function evaluate the results of the test and return 0 if the test
 * failed, and nonzero value if the test passed.
 */
static int
eval_vmalloc_result (struct vparam * param, int result, ...)
{
	return (result == param->result);
} /* eval_vmalloc_result */

static int
eval_vmalloc_from_eq (struct vparam * param, int result, ...)
{
	va_list	args;
	void *	from;

	va_start (args, result);
	from = va_arg (args, void *);
	va_end (args);

	// check the result address besides the result
	return eval_vmalloc_result (param, result, from) && (from == param->from);


} /* eval_vmalloc_from_eq */

static int
eval_vmalloc_from_ge (struct vparam * param, int result, ...)
{
	va_list	args;
	void *	from;

	va_start (args, result);
	from = va_arg (args, void *);
	va_end (args);

	// check the result address besides the result
	return eval_vmalloc_result (param, result, from) && (from >= param->from);
} /* eval_vmalloc_from_ge */

static int
eval_vmalloc_from_lt (struct vparam * param, int result, ...)
{
	va_list	args;
	void *	from;

	va_start (args, result);
	from = va_arg (args, void *);
	va_end (args);

	// check the result address besides the result
	return eval_vmalloc_result (param, result, from) && (from < param->from);
} /* eval_vmalloc_from_lt */


/**
 * vmalloc_test
 * @param:	parameters for the vmalloc call
 *
 * Performs a vmalloc call with parameters @param and checks the result.
 * Returns nonzero value if the call succeeded, 0 if the call failed.
 */
static int
vmalloc_test (struct vparam * param)
{
	int 	result;
	void *	from;

	from = param->from;
	printk ("Allocate %x bytes at %p (%s)\n",
		param->size, param->from, param->desc);

	result = vmalloc (& from, param->size, param->flags);
	printk ("\texpected result: %s, vmalloc result: %s, from: %p\n",
		result_name (param->result), result_name (result), from);

	return param->eval (param, result, from);
} /* vmalloc_test */


/**
 * vfree_test
 * @param:	parameters for the vfree call
 *
 * Performs a vfree call with parameters @param and checks the result.
 * Returns nonzero value if the call succeeded, 0 if the call failed.
 */
static int
vfree_test (struct vparam * param)
{
	int 	result;

	printk ("Free area at %p (%s)\n", param->from, param->desc);

	result = vfree (param->from);
	printk ("\texpected result: %s, vfree result: %s\n",
		result_name (param->result), result_name (result));

	return (result == param->result);
} /* vfree_test */


/**
 * vresize_test
 * @param:	parameters for the vresize call
 *
 * Performs a vresize call with parameters @param and checks the result.
 * Returns nonzero value if the call succeeded, 0 if the call failed.
 */
static int
vresize_test (struct vparam * param)
{
	int 	result;

	printk ("Resize area at %p to %x bytes (%s)\n", param->from,
		param->size, param->desc);

	result = vresize (param->from, param->size);
	printk ("\texpected result: %s, vresize result: %s\n",
		result_name (param->result), result_name (result));

	return (result == param->result);
} /* vresize_test */


/**
 * vmerge_test
 * @param:	parameters for the vmerge call
 *
 * Performs a vmerge call with parameters @param and checks the result.
 * Returns nonzero value if the call succeeded, 0 if the call failed.
 */
static int
vmerge_test (struct vparam * param)
{
	int 	result;

	printk ("Merge areas at %p and %p (%s)\n", param->area1,
		param->area2, param->desc);

	result = vmerge (param->area1, param->area2);
	printk ("\texpected result: %s, vmerge result: %s\n",
		result_name (param->result), result_name (result));

	return (result == param->result);
} /* vmerge_test */


/**
 * vsplit_test
 * @param:	parameters for the vsplit call
 *
 * Performs a vsplit call with parameters @param and checks the result.
 * Returns nonzero value if the call succeeded, 0 if the call failed.
 */
static int
vsplit_test (struct vparam * param)
{
	int 	result;

	printk ("Split area %p at %p (%s)\n", param->from,
		param->split, param->desc);

	result = vsplit (param->from, param->split);
	printk ("\texpected result: %s, vsplit result: %s\n",
		result_name (param->result), result_name (result));

	return (result == param->result);
} /* vsplit_test */


/**
 * vremap_test
 * @param:	parameters for the vremap call
 *
 * Performs a vremap call with parameters @param and checks the result.
 * Returns nonzero value if the call succeeded, 0 if the call failed.
 */
static int
vremap_test (struct vparam * param)
{
	int 	result;

	printk ("Remap area from %p to %p (%s)\n", param->from,
		param->to, param->desc);

	result = vremap (param->from, param->to);
	printk ("\texpected result: %s, vremap result: %s\n",
		result_name (param->result), result_name (result));

	return (result == param->result);
} /* vremap_test */


/**
 * vaccess_test
 * @param:	parameters for the memory access
 *
 * Accesses the memory described in @param.
 * Returns nonzero value to indicate success.
 */
static int
vaccess_test (struct vparam * param)
{
	printk ("Writing %x bytes to area at %p (%s)\n", param->size,
		param->from, param->desc);

	fillmem (param->from, param->size);
	return 1;
} /* vaccess_test */


/**
 * test_sequence
 * @param:	an array of test parameters
 * @ntests:	number of test parameters
 *
 * Performs a sequence of tests. Returns 0 at first failure, nonzero value
 * if all tests passed.
 */
static int
test_sequence (struct vparam * param, int ntests)
{
	while (ntests-- > 0) {
		int passed;

		passed = param->test (param);
		printk ("\t%s\n\n", passed ? "PASSED" : "FAILED");

		if (!passed)
			return 0;

		param++;
	}

	return 1;
} /* test_sequence */



/*
 * Area test sequence.
 */
static struct vparam	area1_seq [] = {
	/*
	* Vmalloc test sequence.
	*/
	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE,
		.size = KUSEG_SIZE,
		.flags = VF_USER_ADDR,
		.result = ENOMEM,
		.eval = eval_vmalloc_result,
		.desc = "exhaust memory"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 1,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "unaligned address"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE,
		.size = 0,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "zero-sized area"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE,
		.size = PAGE_SIZE - 1,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "unaligned size"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KSEG0_BASE - PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "KUSEG->KSEG0 overflow"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KSEG3_BASE + KSEG3_SIZE - PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "KSEG3->KUSEG overflow"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "A at KUSEG_BASE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE,
		.size = 2 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill A"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "B at KUSEG_BASE + 4 * PAGE_SIZE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill B"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 6 * PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "C at KUSEG_BASE + 6 * PAGE_SIZE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE + 6 * PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill C"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KSEG0_BASE - PAGE_SIZE,
		.size = PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "F at KSEG0_BASE - PAGE_SIZE [KUSEG]"
	},

	{
		.test = vaccess_test,
		.from = (void *) KSEG0_BASE - PAGE_SIZE,
		.size = PAGE_SIZE,
		.result = EOK,
		.desc = "fill F"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KSEG3_BASE - PAGE_SIZE,
		.size = PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "G at KSEG3_BASE - PAGE_SIZE [KSEG2]"
	},

	{
		.test = vaccess_test,
		.from = (void *) KSEG3_BASE - PAGE_SIZE,
		.size = PAGE_SIZE,
		.result = EOK,
		.desc = "fill G"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KSEG3_BASE,
		.size = PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "H at KSEG3_BASE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KSEG3_BASE,
		.size = PAGE_SIZE,
		.result = EOK,
		.desc = "fill H"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE - PAGE_SIZE,
		.size = PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "I at KUSEG_BASE - PAGE_SIZE [KSEG3]"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE - PAGE_SIZE,
		.size = PAGE_SIZE,
		.result = EOK,
		.desc = "fill I"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.size = 4 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "full B, C overlap"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "partial B overlap"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.size = 4 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "full B, partial C overlap"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 1 * PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "partial A overlap"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 1 * PAGE_SIZE,
		.size = 4 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_result,
		.desc = "partial A, B overlap"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 8 * PAGE_SIZE,
		.size = 4 * PAGE_SIZE,
		.flags = VF_AUTO_KUSEG,
		.result = EOK,
		.eval = eval_vmalloc_from_ge,
		.desc = "D at/after KUSEG_BASE + 8 * PAGE_SIZE"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KSEG0_BASE - PAGE_SIZE,
		.size = 4 * PAGE_SIZE,
		.flags = VF_AUTO_KUSEG,
		.result = EOK,
		.eval = eval_vmalloc_from_lt,
		.desc = "E before KSEG0_BASE - PAGE_SIZE [KUSEG]"
	},


	/*
	 * Vfree test sequence.
	 */
	{
		.test = vfree_test,
		.from = (void *) KSSEG_BASE,
		.result = EINVAL,
		.desc = "nonexistent area"
	},

	{
		.test = vfree_test,
		.from = (void *) KUSEG_BASE + PAGE_SIZE - 1,
		.result = EINVAL,
		.desc = "unaligned address"
	},

	{
		.test = vfree_test,
		.from = (void *) KUSEG_BASE + PAGE_SIZE,
		.result = EINVAL,
		.desc = "not start of A"
	},

	{
		.test = vfree_test,
		.from = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.result = EOK,
		.desc = "B at KUSEG_BASE + 4 * PAGE_SIZE"
	},

	{
		.test = vfree_test,
		.from = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "nonexistent area"
	},

	/*
	 * Vresize test sequence.
	 */
	{
		.test = vresize_test,
		.from = (void *) KSSEG_BASE,
		.size = 2 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "nonexistent area"
	},

	{
		.test = vresize_test,
		.from = (void *) KUSEG_BASE,
		.size = 0,
		.result = EINVAL,
		.desc = "zero-sized area"
	},

	{
		.test = vresize_test,
		.from = (void *) KUSEG_BASE,
		.size = PAGE_SIZE - 1,
		.result = EINVAL,
		.desc = "unaligned size"
	},

	{
		.test = vresize_test,
		.from = (void *) KUSEG_BASE + PAGE_SIZE,
		.size = PAGE_SIZE,
		.result = EINVAL,
		.desc = "not start of A"
	},

	{
		.test = vresize_test,
		.from = (void *) KUSEG_BASE,
		.size = 8 * PAGE_SIZE,
		.result =  EINVAL,
		.desc = "A overflow to C"
	},

	{
		.test = vresize_test,
		.from = (void *) KSEG0_BASE - PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "F overflow KUSEG->KSEG0"
	},

	{
		.test = vresize_test,
		.from = (void *) KUSEG_BASE - PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "G overflow KSEG3->KUSEG"
	},

	{
		.test = vresize_test,
		.from = (void *) KUSEG_BASE,
		.size = 5 * PAGE_SIZE,
		.result = EOK,
		.desc = "grow A"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE,
		.size = 5 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill A"
	},

	{
		.test = vresize_test,
		.from = (void *) KUSEG_BASE,
		.size = 3 * PAGE_SIZE,
		.result = EOK,
		.desc = "shrink A"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE,
		.size = 3 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill A"
	},

	{
		.test = vresize_test,
		.from = (void *) KUSEG_BASE,
		.size = 3 * PAGE_SIZE,
		.result = EOK,
		.desc = "same A"
	},

	/*
	 * Vmerge test sequence.
	 */
	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.size = 3 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "B at KUSEG_BASE + 3 * PAGE_SIZE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.size = 3 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill B"
	},

	{
		.test = vmerge_test,
		.area1 = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.area2 = (void *) KUSEG_BASE + 6 * PAGE_SIZE,
		.result = EOK,
		.desc = "B + C -> BC at KUSEG_BASE + 3 * PAGE_SIZE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.size = 5 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill BC"
	},

	{
		.test = vfree_test,
		.from = (void *) KUSEG_BASE + 6 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "not start of BC"
	},

	{
		.test = vfree_test,
		.from = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.result = EOK,
		.desc = "BC at KUSEG_BASE + 3 * PAGE_SIZE"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "C at KUSEG_BASE + 4 * PAGE_SIZE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill C"
	},

	{
		.test = vmerge_test,
		.area1 = (void *) KUSEG_BASE,
		.area2 = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "A, C not adjacent"
	},

	{
		.test = vmerge_test,
		.area1 = (void *) KUSEG_BASE,
		.area2 = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "nonexistant second area"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.size = 2 * PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EINVAL,
		.eval = eval_vmalloc_from_eq,
		.desc = "C overlap"
	},

	{
		.test = vmalloc_test,
		.from = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.size = PAGE_SIZE,
		.flags = VF_USER_ADDR,
		.result = EOK,
		.eval = eval_vmalloc_from_eq,
		.desc = "B at KUSEG_BASE + 3 * PAGE_SIZE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.size = PAGE_SIZE,
		.result = EOK,
		.desc = "fill B"
	},

	{
		.test = vmerge_test,
		.area1 = (void *) KUSEG_BASE,
		.area2 = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "A, C not adjacent"
	},

	{
		.test = vmerge_test,
		.area1 = (void *) KUSEG_BASE + 3 * PAGE_SIZE,
		.area2 = (void *) KUSEG_BASE,
		.result = EOK,
		.desc = "A + B -> AB at KUSEG_BASE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE,
		.size = 4 *  PAGE_SIZE,
		.result = EOK,
		.desc = "fill AB"
	},

	{
		.test = vmerge_test,
		.area1 = (void *) KSEG3_BASE - PAGE_SIZE,
		.area2 = (void *) KSEG3_BASE,
		.result = EINVAL,
		.desc = "KSEG2->KSEG3 overflow"
	},

	{
		.test = vresize_test,
		.from = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.size = 4 * PAGE_SIZE,
		.result = EOK,
		.desc = "grow C"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.size = 4 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill C"
	},

	{
		.test = vmerge_test,
		.area1 = (void *) KUSEG_BASE + 4 * PAGE_SIZE,
		.area2 = (void *) KUSEG_BASE,
		.result = EOK,
		.desc = "AB + C -> ABC at KUSEG_BASE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KUSEG_BASE,
		.size = 4 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill ABC"
	},

	{
		.test = vsplit_test,
		.from = (void *) KUSEG_BASE,
		.split = (void *) KUSEG_BASE + 8 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "behind ABC"
	},

	{
		.test = vsplit_test,
		.from = (void *) KUSEG_BASE,
		.split = (void *) KUSEG_BASE,
		.result = EINVAL,
		.desc = "zero-size split"
	},

	{
		.test = vsplit_test,
		.from = (void *) KUSEG_BASE,
		.split = (void *) KUSEG_BASE + PAGE_SIZE / 2,
		.result = EINVAL,
		.desc = "unaligned split"
	},

	{
		.test = vsplit_test,
		.from = (void *) KUSEG_BASE,
		.split = (void *) KUSEG_BASE + 2 * PAGE_SIZE,
		.result = EOK,
		.desc = "ABC at KUSEG_BASE -> A + BC"
	},

	{
		.test = vsplit_test,
		.from = (void *) KUSEG_BASE,
		.split = (void *) KUSEG_BASE + 2 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "behind A"
	},

	{
		.test = vfree_test,
		.from = (void *) KUSEG_BASE,
		.result = EOK,
		.desc = "free A"
	},

	{
		.test = vremap_test,
		.from = (void *) KUSEG_BASE + 2 * PAGE_SIZE,
		.to = (void *) KSSEG_BASE,
		.result = EOK,
		.desc = "C to KSSEG_BASE"
	},

	{
		.test = vaccess_test,
		.from = (void *) KSSEG_BASE,
		.size = 4 * PAGE_SIZE,
		.result = EOK,
		.desc = "fill C"
	},

	{
		.test = vfree_test,
		.from = (void *) KUSEG_BASE + 2 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "C not there"
	},

	{
		.test = vremap_test,
		.from = (void *) KSSEG_BASE,
		.to = (void *) KSEG3_BASE - 2 * PAGE_SIZE,
		.result = EINVAL,
		.desc = "collision with G"
	}

}; /* area1_seq */




void
assignment_test (void)
{
	int result;

	printk (desc);

	/*
	 * Test sequence.
	 */
	result = test_sequence (area1_seq, sizeof_array (area1_seq));
	printk ("Test %s...\n", result ? "passed" : "failed");
} /* assignment_test */

